==> iwrite-1.0.1.4.tar.gz: iwrite-1.0.1.4/src/label_pdf.c <==
/* 
vim:expandtab:softtabstop=2:tabstop=2:shiftwidth=2:nowrap:ruler 
*/
/*
Copyright (c) 2015-2016, iwrite authors 
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
#include "pdf.h"

static int                              g_label_cancel= 0;

struct run_dialog
{
  GtkProgressBar*                       m_progressbar;
  GtkLabel*                             m_file_label;
  GtkButton*                            m_cancel_button;
  size_t                                m_tuple;
};

static gboolean
label_dialog_end(
  gpointer                              i_data)
{
  struct run_dialog*                    l_rd;

  l_rd= (struct run_dialog*)i_data;
  gtk_button_set_label((*l_rd).m_cancel_button, "Ok");
  g_label_cancel= 1;

  return G_SOURCE_REMOVE;
}

static gboolean
label_dialog_update(
  gpointer                              i_data)
{
  char                                  l_text[1+31];
  struct run_dialog*                    l_rd;

  if (0 == g_label_cancel)
  {
    l_rd= (struct run_dialog*)i_data;
    gtk_progress_bar_pulse((*l_rd).m_progressbar);
    sprintf(l_text, "%ld", (*l_rd).m_tuple);
    gtk_label_set_text((*l_rd).m_file_label, l_text);
  }

  return G_SOURCE_REMOVE;
}

static int
label_run_property_bind(
  struct run_dialog*const               io_rd,
  GtkBuilder*const                      i_builder)
{
  int                                   l_exit;

  l_exit= 0;

  do
  {

    (*io_rd).m_progressbar= GTK_PROGRESS_BAR(
      gtk_builder_get_object(i_builder, "progressbar1"));

    (*io_rd).m_file_label= GTK_LABEL(
      gtk_builder_get_object(i_builder, "file_label"));

    (*io_rd).m_cancel_button= GTK_BUTTON(
      gtk_builder_get_object(i_builder, "cancel_button"));

  }while(0);

  return l_exit;
}

static int
label_run_loop(
  GError**                              o_error,
  struct run_dialog*const               io_dialog,
  struct rw*const                       io_rw,
  struct report *const                  io_report)
{
  unsigned                              l_col;
  int                                   l_eof;
  int                                   l_exit;
  struct label*                         l_label;
  struct label_paper const*             l_paper;
  unsigned                              l_row;
  gint64                                l_time1;
  gint64                                l_time2;

  l_exit= 0;
  l_col= 0;
  l_row= 0;
  l_time1= g_get_monotonic_time();
  l_label= &(*io_report).m_object.m_label;
  l_paper= &(*l_label).m_config.m_paper;

  rw_page_clear_accumulator((*l_label).m_page);

  (*io_dialog).m_tuple= 1;

  (*io_rw).m_pos_x= (*l_paper).m_left_margin;
  (*io_rw).m_pos_y= (*l_paper).m_top_margin;

  do
  {

    rw_page_set_value((*l_label).m_page, io_rw);
    cairo_save((*io_rw).m_cr);
    cairo_translate((*io_rw).m_cr, (*io_rw).m_pos_x, (*io_rw).m_pos_y);
    iwr_page_draw_item((*io_rw).m_cr, (*l_label).m_page, 0);
    cairo_restore((*io_rw).m_cr);

    l_col++;

    if ((*l_paper).m_columns > l_col)
    {
      (*io_rw).m_pos_x+= (*l_paper).m_horizontal_pitch;
    }
    else
    {
      l_col= 0;
      l_row++;
      (*io_rw).m_pos_x= (*l_paper).m_left_margin;

      if ((*l_paper).m_rows > l_row)
      {
        (*io_rw).m_pos_y+= (*l_paper).m_vertical_pitch;
      }
      else
      {
        cairo_show_page((*io_rw).m_cr);
        l_row= 0;
        (*io_rw).m_pos_y= (*l_paper).m_top_margin;
        (*io_rw).m_page_number++;
      }
    }

    if (g_label_cancel)
    {
      cairo_show_page((*io_rw).m_cr);
      break;
    }

    l_exit= query_exec_next(o_error, &l_eof, &(*io_rw).m_exec);

    if (l_exit)
    {
      cairo_show_page((*io_rw).m_cr);
      break;
    }

    if (l_eof)
    {
      cairo_show_page((*io_rw).m_cr);
      break;
    }

    (*io_dialog).m_tuple++;

    if ((*io_dialog).m_progressbar)
    {
      l_time2= g_get_monotonic_time();
      if (100000 < (l_time2 - l_time1))
      {
        l_time1= l_time2;
        gdk_threads_add_idle(label_dialog_update, io_dialog);
      }
    }

  }while(1);

  if ((*io_dialog).m_progressbar)
  {
    label_dialog_update(io_dialog);
  }

  return l_exit;
}

static int
label_run_exec(
  struct run_dialog*const               io_dialog,
  struct report *const                  io_report,
  struct goop const*const               i_goop)
{ 
  GtkDialog*                            l_dialog;
  int                                   l_eof;
  GError*                               l_error;
  int                                   l_exit;
  struct label*                         l_label;
  struct rw                             l_rw;

  l_dialog= 0;
  l_error=0;
  l_exit= 0;
  l_label= &(*io_report).m_object.m_label;
  rw_assign(&l_rw);

  do
  {
  
    l_exit= rw_cairo_pdf_surface_create(
      &l_error,
      &l_rw,
      (*l_label).m_config.m_paper.m_page_width,
      (*l_label).m_config.m_paper.m_page_height,
      (*i_goop).m_outputfile);

    if (l_exit)
    {
      break;
    }

    l_exit= query_exec_connect_prepare_first(
      &l_error,
      &l_eof,
      &l_rw.m_exec,
      &(*io_report).m_query,
      &(*io_report).m_aspect,
      (*i_goop).m_prompt,
      (*i_goop).m_interactive);

    if ((l_exit) || (l_eof))
    {
      cairo_show_page(l_rw.m_cr);
      break;
    }

    label_run_loop(&l_error, io_dialog, &l_rw, io_report);

  }while(0);

  if (l_error)
  {
    if ((*i_goop).m_interactive)
    {
      _error_display_prompt(l_error);
    }
  }

  g_clear_error(&l_error);

  rw_discharge(&l_rw);

  return l_exit;
}

struct label_thread_data
{
  struct run_dialog*                    m_dialog;
  struct report*                        m_report;
  struct goop const*                    m_goop;
};

static gpointer
label_run_thread(
  gpointer                              i_data)
{
  int                                   l_exit;
  struct label_thread_data*               l_info;

  l_info= (struct label_thread_data*)i_data;

  l_exit= label_run_exec(
    (*l_info).m_dialog,
    (*l_info).m_report,
    (*l_info).m_goop);

  if (0 == g_label_cancel)
  {
    gdk_threads_add_idle(label_dialog_end, (*l_info).m_dialog);
    while(0 == g_label_cancel);
  }

  g_thread_exit((gpointer)(ptrdiff_t)l_exit);

  return (gpointer)(ptrdiff_t)l_exit;
}

extern int
label_pdf_run(
  struct report *const                  io_report,
  struct goop const*const               i_goop)
{
  GtkBuilder*                           l_builder;
  GtkDialog*                            l_dialog;
  GError*                               l_error;
  int                                   l_exit;
  struct label_thread_data              l_info;
  int                                   l_rc;
  struct run_dialog                     l_rd;
  GThread*                              l_thread;

  l_builder= 0;
  l_dialog= 0;
  l_error= 0;
  l_exit= 0;
  l_thread= 0;
  memset(&l_rd, 0, sizeof(l_rd));
  memset(&l_info, 0, sizeof(l_info));

  do
  {

    g_label_cancel= 0;

    if ((*i_goop).m_interactive)
    {
      database_aspect_password_fill(&g_label_cancel, &(*io_report).m_aspect);

      if (g_label_cancel)
      {
        break;
      }
    }
    else
    {
      l_exit= label_run_exec(&l_rd, io_report, i_goop);
      break;
    }

    l_info.m_dialog= 0;
    l_info.m_report= io_report;
    l_info.m_goop= i_goop;

    l_builder= gtk_builder_new();
    l_rc= wrap_gtk_builder_add_from_file(l_builder, "runreport.glade", &l_error);

    if (0 == l_rc)
    {
      _error_log(l_error);
      g_warning("Unable to build dialog: %s\n", l_error->message);
      l_exit= -1;
      break;
    }

    l_dialog= (GtkDialog*)gtk_builder_get_object(l_builder, "run_dialog");

    if (0 == l_dialog)
    {
      g_warning("Unable to find dialog object: 'run_dialog'");
      l_exit= -1;
      break;
    }

    label_run_property_bind(&l_rd, l_builder);

    gtk_window_set_transient_for(GTK_WINDOW(l_dialog), mainw());
    gtk_widget_show_all(GTK_WIDGET(l_dialog));
    gtk_window_set_modal(GTK_WINDOW(l_dialog), 1);

    gtk_widget_grab_focus(gtk_dialog_get_widget_for_response(
      GTK_DIALOG(l_dialog), GTK_RESPONSE_CANCEL));

    l_info.m_dialog= &l_rd;
    l_thread= g_thread_new("label", label_run_thread, &l_info);

    gtk_dialog_run(l_dialog);

    g_label_cancel= 1;
    g_thread_join(l_thread);

  }while(0);

  if (l_builder)
  {
    g_object_unref(l_builder);
  }

  g_clear_error(&l_error);

  if (l_dialog)
  {
    gtk_widget_destroy(GTK_WIDGET(l_dialog));
  }

  if (l_thread)
  {
    g_thread_unref(l_thread);
  }

  return l_exit;
}

==> iwrite-2.0.1.0.tar.gz: iwrite-2.0.1.0/src/label_pdf.c <==
/* 
vim:expandtab:softtabstop=2:tabstop=2:shiftwidth=2:nowrap:ruler 
*/
/*
Copyright (c) 2015-2016, iwrite authors 
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
#include "label_pdf.h"

static int                              g_label_cancel= 0;

struct run_dialog
{
  GtkProgressBar*                       m_progressbar;
  GtkLabel*                             m_file_label;
  GtkButton*                            m_cancel_button;
  size_t                                m_tuple;
};

static gboolean
label_dialog_end(
  gpointer                              i_data)
{
  struct run_dialog*                    l_rd;

  l_rd= (struct run_dialog*)i_data;
  gtk_button_set_label((*l_rd).m_cancel_button, "Ok");
  g_label_cancel= 1;

  return G_SOURCE_REMOVE;
}

static gboolean
label_dialog_update(
  gpointer                              i_data)
{
  char                                  l_text[1+31];
  struct run_dialog*                    l_rd;

  if (0 == g_label_cancel)
  {
    l_rd= (struct run_dialog*)i_data;
    gtk_progress_bar_pulse((*l_rd).m_progressbar);
    snprintf(l_text, sizeof(l_text), "%ld", (*l_rd).m_tuple);
    gtk_label_set_text((*l_rd).m_file_label, l_text);
  }

  return G_SOURCE_REMOVE;
}

static int
label_run_property_bind(
  struct run_dialog*const               io_rd,
  GtkBuilder*const                      i_builder)
{
  int                                   l_exit;

  l_exit= 0;

  do
  {

    (*io_rd).m_progressbar= GTK_PROGRESS_BAR(
      gtk_builder_get_object(i_builder, "progressbar1"));

    (*io_rd).m_file_label= GTK_LABEL(
      gtk_builder_get_object(i_builder, "file_label"));

    (*io_rd).m_cancel_button= GTK_BUTTON(
      gtk_builder_get_object(i_builder, "cancel_button"));

  }while(0);

  return l_exit;
}

static int
label_run_loop(
  GError**                              o_error,
  struct run_dialog*const               io_dialog,
  struct rw*const                       io_rw,
  struct label_spec *const              io_label)
{
  unsigned                              l_col;
  int                                   l_eof;
  int                                   l_exit;
  unsigned                              l_row;
  gint64                                l_time1;
  gint64                                l_time2;

  l_exit= 0;
  l_col= 0;
  l_row= 0;
  l_time1= g_get_monotonic_time();

  rw_item_clear_accumulator(&(*io_label).m_report.m_item);

  (*io_dialog).m_tuple= 1;

  (*io_rw).m_pos_x= (*io_label).m_paper.m_left_margin;
  (*io_rw).m_pos_y= (*io_label).m_paper.m_top_margin;

  do
  {

    rw_item_set_value(&(*io_label).m_report.m_item, io_rw);
    cairo_save((*io_rw).m_cr);
    cairo_translate((*io_rw).m_cr, (*io_rw).m_pos_x, (*io_rw).m_pos_y);
    item_draw(&(*io_label).m_report.m_item, (*io_rw).m_cr, 0);
    cairo_restore((*io_rw).m_cr);

    l_col++;

    if ((*io_label).m_paper.m_columns > l_col)
    {
      (*io_rw).m_pos_x+= (*io_label).m_paper.m_horizontal_pitch;
    }
    else
    {
      l_col= 0;
      l_row++;
      (*io_rw).m_pos_x= (*io_label).m_paper.m_left_margin;

      if ((*io_label).m_paper.m_rows > l_row)
      {
        (*io_rw).m_pos_y+= (*io_label).m_paper.m_vertical_pitch;
      }
      else
      {
        cairo_show_page((*io_rw).m_cr);
        l_row= 0;
        (*io_rw).m_pos_y= (*io_label).m_paper.m_top_margin;
        (*io_rw).m_page_number++;
      }
    }

    if (g_label_cancel)
    {
      cairo_show_page((*io_rw).m_cr);
      break;
    }

    l_exit= query_exec_next(o_error, &l_eof, &(*io_rw).m_exec);

    if (l_exit)
    {
      cairo_show_page((*io_rw).m_cr);
      break;
    }

    if (l_eof)
    {
      cairo_show_page((*io_rw).m_cr);
      break;
    }

    (*io_dialog).m_tuple++;

    if ((*io_dialog).m_progressbar)
    {
      l_time2= g_get_monotonic_time();
      if (100000 < (l_time2 - l_time1))
      {
        l_time1= l_time2;
        gdk_threads_add_idle(label_dialog_update, io_dialog);
      }
    }

  }while(1);

  if ((*io_dialog).m_progressbar)
  {
    label_dialog_update(io_dialog);
  }

  return l_exit;
}

static int
label_run_exec(
  struct run_dialog*const               io_dialog,
  struct label_spec *const              io_label,
  struct goop const*const               i_goop)
{ 
  int                                   l_eof;
  GError*                               l_error;
  int                                   l_exit;
  struct rw                             l_rw;

  l_error=0;
  l_exit= 0;
  rw_assign(&l_rw);

  do
  {
  
    l_exit= rw_cairo_pdf_surface_create(
      &l_error,
      &l_rw,
      (*io_label).m_paper.m_page_width,
      (*io_label).m_paper.m_page_height,
      (*i_goop).m_outputfile);

    if (l_exit)
    {
      break;
    }

    l_exit= query_exec_connect_prepare_first(
      &l_error,
      &l_eof,
      &l_rw.m_exec,
      &(*io_label).m_query,
      &(*io_label).m_aspect,
      (*i_goop).m_prompt,
      (*i_goop).m_interactive);

    if ((l_exit) || (l_eof))
    {
      cairo_show_page(l_rw.m_cr);
      break;
    }

    label_run_loop(&l_error, io_dialog, &l_rw, io_label);

  }while(0);

  if (l_error)
  {
    if ((*i_goop).m_interactive)
    {
      _error_display_prompt(l_error);
    }
  }

  g_clear_error(&l_error);

  rw_discharge(&l_rw);

  return l_exit;
}

struct label_thread_data
{
  struct run_dialog*                    m_dialog;
  struct label_spec*                    m_label;
  struct goop const*                    m_goop;
};

static gpointer
label_run_thread(
  gpointer                              i_data)
{
  int                                   l_exit;
  struct label_thread_data*             l_info;

  l_info= (struct label_thread_data*)i_data;

  l_exit= label_run_exec(
    (*l_info).m_dialog,
    (*l_info).m_label,
    (*l_info).m_goop);

  if (0 == g_label_cancel)
  {
    gdk_threads_add_idle(label_dialog_end, (*l_info).m_dialog);
    while(0 == g_label_cancel);
  }

  g_thread_exit((gpointer)(ptrdiff_t)l_exit);

  return (gpointer)(ptrdiff_t)l_exit;
}

extern int
label_pdf_run(
  struct label_spec *const              io_label,
  struct goop const*const               i_goop)
{
  GtkBuilder*                           l_builder;
  GtkDialog*                            l_dialog;
  GError*                               l_error;
  int                                   l_exit;
  struct label_thread_data              l_info;
  int                                   l_rc;
  struct run_dialog                     l_rd;
  GThread*                              l_thread;

  l_builder= 0;
  l_dialog= 0;
  l_error= 0;
  l_exit= 0;
  l_thread= 0;
  memset(&l_rd, 0, sizeof(l_rd));
  memset(&l_info, 0, sizeof(l_info));

  do
  {

    g_label_cancel= 0;

    if ((*i_goop).m_interactive)
    {
      database_aspect_password_fill(&g_label_cancel, &(*io_label).m_aspect);

      if (g_label_cancel)
      {
        break;
      }
    }
    else
    {
      l_exit= label_run_exec(&l_rd, io_label, i_goop);
      break;
    }

    l_info.m_dialog= 0;
    l_info.m_label= io_label;
    l_info.m_goop= i_goop;

    l_builder= gtk_builder_new();
    l_rc= wrap_gtk_builder_add_from_file(l_builder, "runreport.glade", &l_error);

    if (0 == l_rc)
    {
      _error_log(l_error);
      g_warning("Unable to build dialog: %s\n", l_error->message);
      l_exit= -1;
      break;
    }

    l_dialog= (GtkDialog*)gtk_builder_get_object(l_builder, "run_dialog");

    if (0 == l_dialog)
    {
      g_warning("Unable to find dialog object: 'run_dialog'");
      l_exit= -1;
      break;
    }

    label_run_property_bind(&l_rd, l_builder);

    gtk_window_set_transient_for(GTK_WINDOW(l_dialog), get_main_window());
    gtk_widget_show_all(GTK_WIDGET(l_dialog));
    gtk_window_set_modal(GTK_WINDOW(l_dialog), 1);

    gtk_widget_grab_focus(gtk_dialog_get_widget_for_response(
      GTK_DIALOG(l_dialog), GTK_RESPONSE_CANCEL));

    l_info.m_dialog= &l_rd;
    l_thread= g_thread_new("label", label_run_thread, &l_info);

    gtk_dialog_run(l_dialog);

    g_label_cancel= 1;
    g_thread_join(l_thread);

  }while(0);

  if (l_builder)
  {
    g_object_unref(l_builder);
  }

  g_clear_error(&l_error);

  if (l_dialog)
  {
    gtk_widget_destroy(GTK_WIDGET(l_dialog));
  }

  if (l_thread)
  {
    g_thread_unref(l_thread);
  }

  return l_exit;
}
